<?php
/**
 * @file
 * Tabular (tabular) administration layer.
 */

/**
 * Module settings.
 *
 * @param array $form
 * @param array &$form_state
 */
function tabular_admin_form($form, &$form_state) {
  $form['#attributes'] = array(
    'autocomplete' => 'off',
  );

  $datatable = variable_get('tabular_library_datatable', '');
  if ($datatable) {
    $list = explode('|', $datatable);
    $datatable_module = $list[0];
    $datatable_library = $list[1];
    unset($list);
  }
  else {
    $datatable_module = $datatable_library = '';
  }

  $form['#attached']['js'][] = drupal_get_path('module', 'tabular') . '/js/tabular.base.js';

  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General settings'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
    'tabular_allow_group_overwrite' => array(
      '#type' => 'checkbox',
      '#title' => t('Allow multiple instances of groups'),
      '#description' => t('Risky and experts only. See \'Multiple instances warning\' in a <em>target</em> field\'s instance settings form.'),
      '#default_value' => variable_get('tabular_allow_group_overwrite', 0),
      '#title_display' => 'before',
    ),
    'tabular_default_css' => array(
      '#type' => 'checkbox',
      '#title' => t('Attach default CSS stylesheet'),
      '#description' => t('Otherwise, check the stylesheet at tabular/css/tabular.suggestions.css and implement equivalent in theme.'),
      '#default_value' => variable_get('tabular_default_css', 1),
      '#title_display' => 'before',
    ),
    'tabular_library_datatable' => array(
      '#type' => 'hidden',
      '#default_value' => $datatable,
    ),
    'datatable' => array(
      '#type' => 'fieldset',
      '#title' => t('Include jQuery.dataTable via Library API'),
      '#tree' => TRUE,
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      'help' => array(
        '#type' => 'markup',
        '#markup' => t('
<p><b>Do we need that?</b>
<br>If one of the tabular fields of this site \'Use dynamic table view\', and jQuery.dataTable isn\'t included by some other means: then <em>yes</em>.
</p>
<p><b>What is it?</b>
<br>It\'s a Javascript feature that makes tables sortable by columns, paginated, and optionally searchable.
<br>In essence it makes tables more user-friendly, particurly when containing large quantities of data.
<br>jquery.dataTable 1.9.4 (<1.1) !description and !reference.
</p>
<p><b>Where can we get jQuery.dataTable - we have no Library API module that provides that</b>
<br>The only supported version is 1.9.4 - later versions aren\'t compatible with the jQuery 1.4.4 bundled with Drupal 7 core.
<br>Download !download or search for \'download jquery datatables 1.9.4\' via a search engine.
<br>Only the Javascript .js is needed - CSS and images aren\'t needed (if you attach the default stylesheet).
</p>
<p><b>This is boring and complicated - why doesn\'t Tabular bundle it</b>
<br>Though jquery.dataTable is released under a Drupal (GNU) compatible license (GPL 2) it is usually not allowed to bundle third party assets with Drupal contrib resources.
<br>See !ban.
</p>
',
          array(
            '!description' => l(
              'Description',
              'https://legacy.datatables.net/',
              array(
                'attributes' => array('target' => '_blank')
              )
            ),
            '!reference' => l(
              'Reference',
              'https://legacy.datatables.net/ref',
              array(
                'attributes' => array('target' => '_blank')
              )
            ),
            '!download' => l(
              'DataTables 1.9.4',
              'https://cdn.datatables.net/1.9.4',
              array(
                'attributes' => array('target' => '_blank')
              )
            ),
            '!ban' => l(
              '3rd party libraries and content on Drupal.org',
              'https://www.drupal.org/node/422996',
              array(
                'attributes' => array('target' => '_blank')
              )
            )
          )
        ),
      ),
      'module' => array(
        '#type' => 'textfield',
        '#title' => t('Library module machine name'),
        '#default_value' => $datatable_module,
        '#states' => array(
          'optional' => array(
            ':input[name="datatable[library]"]' => array('value' => ''),
          ),
        ),
        '#size' => 30,
        '#prefix' => '<div><div style="float:left; margin-top:0; padding-top:0">',
        '#suffix' => '</div>',
      ),
      'library' => array(
        '#type' => 'textfield',
        '#title' => t('Library name'),
        '#default_value' => $datatable_library,
        '#states' => array(
          'optional' => array(
            ':input[name="datatable[module]"]' => array('value' => ''),
          ),
        ),
        '#size' => 30,
        '#prefix' => '<div style="float:left; margin-left:2em; margin-top:0; padding-top:0">',
        '#suffix' => '</div><div style="clear:both"/></div>',
      ),
    ),
    'tabular_converter_format' => array(
      '#type' => 'fieldset',
      '#title' => t('JS converter formats by language'),
      //'#tree' => TRUE,
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      'help' => array(
        '#type' => 'markup',
        '#markup' => t('
<p><b>Time</b>
<br>Use !unix_time_chars (+ uppercase C for ISO-8601 Z; UTC, no timezone), and (optionally) non-letter separators.
<br>Supported converting <em>from</em> locale: r, c, C, Y, m, d, H, i, s, u, U.
<br>Supported converting <em>to</em> locale: r, c, C, Y, y, m, n, d, j, H, h, G, g, i, s, u, U.
</p>
<p><b>Default formats</b>
<br>U.S. English.
<br>For non-U.S. English (and lots of other locales), dates are usually day-month-year; <em>not</em> (U.S.) month-day-year.
</p>
',
          array(
            '!unix_time_chars' => l(
              'PHP\'s version of Unix time characters',
              'http://php.net/manual/en/function.date.php',
              array(
                'attributes' => array('target' => '_blank')
              )
            ),
            '!reference' => l(
              'Reference',
              'https://legacy.datatables.net/ref',
              array(
                'attributes' => array('target' => '_blank')
              )
            ),
            '!download' => l(
              'DataTables 1.9.4',
              'https://cdn.datatables.net/1.9.4',
              array(
                'attributes' => array('target' => '_blank')
              )
            ),
            '!ban' => l(
              '3rd party libraries and content on Drupal.org',
              'https://www.drupal.org/node/422996',
              array(
                'attributes' => array('target' => '_blank')
              )
            )
          )
        ),
      ),
    ),
  );
  $langs = language_list();
  $lang_codes = array_keys($langs);
  $fields_format =& $form['general']['tabular_converter_format'];
  $format_default = tabular_converter_format_default();
  $formats = array();
  inspect(date('c'));
  foreach ($lang_codes as $code) {
    $formats[$code] = variable_get('tabular_converter_format_' . $code, $format_default);
    $format =& $formats[$code];
    $fields_format['tabular_converter_format_' . $code] = array(
      '#type' => 'fieldset',
      '#title' => $code . ': ' . $langs[$code]->name . ' (' . $langs[$code]->native . ')',
      '#tree' => TRUE,
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    foreach ($format_default as $name => $list) {
      switch ($name) {
        case 'number':
          $title = t('Number');
          break;
        case 'money':
          $title = t('Money');
          break;
        case 'time':
          $title = t('Time');
          break;
        default:
          $title = $name;
      }
      $fields_format['tabular_converter_format_' . $code][$name] = array(
        '#type' => 'fieldset',
        '#title' => $title . ' <span style="text-transform:none">[' . $name . ']</span>',
        '#tree' => TRUE,
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
      );
      // @todo: Use array_merge_recursive() instead, to support custom formats.
      if (!isset($format[$name])) {
        $format[$name] = $list;
      }
      foreach ($list as $sub => $val) {
        $required = TRUE;
        $description = '';
        switch ($sub) {
          case 'thousand_sep':
            $required = FALSE;
            $title = t('Thousand separator') . ' [' . $sub . ']';
            if ($name == 'money') {
              $description = t('Money must/will have same decimal mark as Number.');
            }
            break;
          case 'decimal_mark':
            $title = t('Decimal mark') . ' [' . $sub . ']';
            break;
          case 'n_decimals':
            $title = t('Number of decimals') . ($name !== 'number' ? '' : (' (' . t('float only') . ')')) . ' [' . $sub . ']';
            break;
          case 'no_fraction':
            $required = FALSE;
            $title = t('No fraction mark') . ' [' . $sub . ']';
            break;
          default:
            $title = $sub;
        }
        $fields_format['tabular_converter_format_' . $code][$name][$sub] = array(
          '#type' => 'textfield',
          '#title' => $title,
          '#default_value' => isset($format[$name][$sub]) ? $format[$name][$sub] : $val,
          '#required' => $required,
          '#size' => 30,
        );
        if ($description) {
          $fields_format['tabular_converter_format_' . $code][$name][$sub]['#description'] = $description;
        }
      }
    }
    unset($format);
  }
  unset($fields_format);

  // #states:required don't seem to work validation-wise.
  $form['#element_validate'][] = 'tabular_admin_form_validate';

  $form['#submit'][] = 'tabular_admin_form_submit';

  return system_settings_form($form);
}

/**
 * @param array &$form
 * @param array &$form_state
 */
function tabular_admin_form_validate($form, &$form_state) {
  $values =& $form_state['values'];
  if (trim($values['datatable']['module'])) {
    if (!trim($values['datatable']['library'])) {
      form_set_error(
        'datatable][library',
        t('!name field is required.', array('!name' => t('Library name')))
      );
    }
  }
  elseif (trim($values['datatable']['library'])) {
    form_set_error(
      'datatable][module',
      t('!name field is required.', array('!name' => t('Library module machine name')))
    );
  }
}

/**
 * @param array &$form
 * @param array &$form_state
 */
function tabular_admin_form_submit($form, &$form_state) {
  $input =& $form_state['input'];
  $values =& $form_state['values'];
  // Join module and library vars into a single var.
  if (($module = trim($values['datatable']['module']))) {
    $input['tabular_library_datatable'] = $values['tabular_library_datatable'] =
      $values['datatable']['module'] . '|' . trim($values['datatable']['library']);
  }
  else {
    $input['tabular_library_datatable'] = $values['tabular_library_datatable'] = '';
  }
  // Don't save module and library vars.
  unset($input['datatable']['module'], $values['datatable']['module']);
  unset($input['datatable']['library'], $values['datatable']['library']);
}
