/**
 * @file
 * Tabular (tabular) admin.
 */

/*jslint browser: true, continue: true, indent: 2, newcap: true, nomen: true, plusplus: true, regexp: true, white: true, ass: true*/
/*global alert: false, confirm: false, console: false*/
/*global jQuery: false, Drupal: false, Judy: false*/

(function($) {
  'use strict';

  var utils = {};
  utils.eventHandlers = {
    /**
     * Machine name: cannot start with a digit.
     * Apply or call: uses this or arg that.
     * @param {Event} [evt]
     * @param {HTMLElement} [that]
     */
    trim: function(evt, that) {
      var self = that || this, v = self.value;
      if (v !== '') {
        self.value = $.trim(v);
      }
    },
    /**
     * Machine name: cannot start with a digit.
     * Apply or call: uses this or arg that.
     * @param {Event} [evt]
     * @param {HTMLElement} [that]
     */
    convertMachineName: function(evt, that) {
      var self = that || this, v = self.value;
      if (v !== '') {
        v = $.trim(v);
        if (v !== '') {
          v = v.toLowerCase().replace(/[^a-z\d_]/g, '_');
          // Prefix underscore if first char is digit.
          if (/\d/.test(v.charAt(0))) {
            v = '_' + v;
          }
        }
        self.value = v;
      }
    },
    //
    /**
     * Lax name: upper + lowercase + dash, and no first char requirements.
     * Apply or call: uses this or arg that.
     * @param {Event} [evt]
     * @param {HTMLElement} [that]
     */
    convertLaxName: function(evt, that) {
      var self = that || this, v = self.value;
      if (v !== '') {
        v = $.trim(v);
        if (v !== '') {
          v = v.replace(/[^a-zA-Z\d_\-]/g, '_');
        }
        self.value = v;
      }
    }
  };

  if (!Drupal.tabular) {
    Drupal.tabular = {};
  }
  Drupal.tabular.field_ui = utils;


  Drupal.behaviors.tabularAdmin = {
    attach: function() {
      var $field, $option;
      // Lock cardinality select to value '1'.
      $field = $('select#edit-field-cardinality').not('locked-field-cardinality');
      if ($field.length) {
        $field.attr('locked-field-cardinality', '1');
        $option = $('option[value="1"]', $field);
        if ($option.length) {
          $option.attr('selected', 'selected');
          $field.empty();
          $field.append($option);
        }
      }

      // Avoid ridiculously large help text field.
      $('textarea[name="instance[description]"]').not('[lessened-field-description]').
        attr('lessened-field-description', '1').attr('rows', '1');
    }
  };

}(jQuery));
