/**
 * @file
 * Tabular (tabular) widget.
 */

/*jslint browser: true, continue: true, indent: 2, newcap: true, nomen: true, plusplus: true, regexp: true, white: true, ass: true*/
/*global alert: false, confirm: false, console: false*/
/*global jQuery: false, Drupal: false, Judy: false*/

(function($) {
  'use strict';

  // Import.
  var Tabular = Drupal.Tabular;

  /**
   * Abstract base class for all solitary widget classes.
   * @see Tabular.base
   * @see Tabular.widget.base
   * @see Tabular.widget.value
   * @see Tabular.widget.source
   * @constructor
   */
  Tabular.widget.solitary = function() {
    throw new Error('Abstract class ' + this.type_of + ' is not to be instantiated.');
  };
  Tabular.widget.solitary.prototype = {
    constructor: Tabular.widget.solitary,
    /**
     * Type flag; to check if current instance extends this class.
     */
    isSolitary: true,
    /**
     * @throws {Error}
     * @returns {object}
     *   Self.
     */
    initSolitary: function() {
      var self = this, errorCode, msg;
      if (self.state) {
        return self;
      }
      try {
        //
        return self;
      }
      catch (er) {
        errorCode = (typeof er.customCode === 'function' ? er.customCode() : 0) || Tabular.errorCode('frontend_init_widget');
        inspect.traceLog(
          er,
          {
            type: self.module || 'tabular',
            message: er.message,
            code: errorCode,
            severity: 'error'
          }
        );
        // Display error to user, unless other previous initialization error.
        if (!self.state) {
          // Initialization error.
          self.state = 1;
          msg = Tabular.message('generic', 'fatal_frontend_error', {'@errorcode': errorCode});
          if (!self.displayErrorMessage(msg)) {
            Tabular.alertDialog(msg, 'error');
          }
        }
        // Re-throw the error but tell caller to abort; don't log (again) etc.
        er.abortSilently = true;
        throw er;
      }
    }
  };
  // base + widget + value + source + solitary.
  Tabular.extend(Tabular.widget.solitary, 'Tabular.widget.solitary',
    Tabular.widget.value, Tabular.widget.source);

}(jQuery));
