<?php
/**
 * @file
 * Tabular (tabular) field formatter hook implementations
 * which extending modules - field types - needn't implement too.
 */

/**
 * Defines (non-form) formatters and which field types of this module may use
 * the formatters.
 *
 * Implements hook_field_formatter_info().
 *
 * @return array
 */
function tabular_field_formatter_info() {
  return array(
    'tabular_view_table' => array(
      'label' => t('Table listing selected rows'),
      'field types' => array(
        'tabular'
      ),
    ),
  );
}

/**
 * @todo: field_formatter_view is completely outdated.
 * @see tabular_field_widget_form()
 *
 * Formats field value for display.
 *
 * Implements hook_field_formatter_view().
 */
function tabular_field_formatter_view($entity_type, $entity, $field, $instance, $langcode, $items, $display) {
  return array();

  $element = array();

  if ($display['type'] == 'tabular_view_table') {
    // There's always only a single value; a JSON-encoded object list.
    // Real cardinality is not supported; only 'mock_cardinality'.
    $value = reset($items);
    if ($value && ($value = $value['value']) && $value !== '{}') {
      $rows = drupal_json_decode($value);
      if ($rows) {
        $le = count($items);

        // Include class handling this tabular entity kind.
        $entity_kind = $field['settings']['entity_kind'];
        $entity_kind_specs = tabular_entity_kind($entity_kind);
        $module = $entity_kind_specs['module'];
        $class = $entity_kind_specs['class'];
        // Include entity kind defining class (and ancestors).
        foreach ($entity_kind_specs['includes'] as $cls => $include) {
          module_load_include($include['ext'], $include['module'], $include['path_file']);
        }
        unset($entity_kind_specs, $cls, $include);

        $column_definitions = call_user_func_array($class . '::getColumnDefinitions', array($entity_kind, 'view'));
        $visible_columns = explode("\n", str_replace(array("\r", ' '), '', $instance['widget']['settings']['visible_columns']));

        for ($i = 0; $i < $le; ++$i) {
          $row = $rows[$i];
          $start = $markup = $end = '';
          if (!$i) {
            $start = '<table class="tabular-view-table"><thead><tr>';
            foreach ($visible_columns as $column) {
              if (empty($column_definitions[$column]['label'])) {
                watchdog(
                  $module,
                  'Missing/empty label for column @column in ' . $class . '::getColumnDefinitions().',
                  array('@column' => $column),
                  WATCHDOG_ERROR
                );
                $label = '_UNDEFINED_';
              }
              else {
                $label = $column_definitions[$column]['label'];
              }
              $start .= '<th class="column-' . $column . '">' . $label . '</th>';
            }
            $start .= '</tr></thead><tbody>';
          }
          elseif ($i == $le - 1) {
            $end = '</tbody></table>';
          }

          $markup = '<tr>';
          foreach ($row as $column => $value) {
            if (in_array($column, $visible_columns)) {
              if (!empty($column_definitions[$column]['format'])) {
                $format = $column_definitions[$column]['format'];
                $format_name = key($format);
                switch ($format_name) {
                  case 'date':
                    // Make column sortable by prefixing sortable value.
                    $formatted = '<span hidden aria-hidden="true">' . $value . '</span>'
                      // Convert iso 8601 YYYY-MM-DD to provided format.
                      . date(reset($format), mktime(12, 1, 1, substr(5, 2, $value), substr(8, 2, $value), substr(0, 4, $value)));
                    break;
                  default:
                    watchdog(
                      $module,
                      'Unsupported format @format of column @column in ' . $class . '::getColumnDefinitions().',
                      array('@format' => $format_name, '@column' => $column),
                      WATCHDOG_ERROR
                    );
                    $formatted = check_plain($value);
                }
              }
              else {
                $formatted = check_plain($value);
              }
              $markup .= '<td class="column-' . $column . '">' . $formatted . '</td>';
            }
          }
          $markup .= '</tr>';

          $element[0] = array(
            '#type' => 'markup',
            '#markup' => $start . $markup . $end,
          );
        }

      }
    }
  }

  return $element;
}
