<?php
/**
 * Tabular Example (tabular_example).
 */

/**
 * Class TabularExample.
 * Defines 'example' entity kind.
 */
class TabularExample extends Tabular implements TabularInterface {

  // Attributes and methods extending Tabular + TabularInterface. --

  /**
   * Name of this module.
   *
   * @type string
   */
  const MODULE = 'tabular_example';

  /**
   * Name entity kind this class declares.
   *
   * @type string
   */
  const ENTITY_KIND = 'example';

  /**
   * Get entity kind schema of this class (entity kind).
   *
   * A list of primary key, view_columns and optionally buttons.
   *
   * NB: the view_columns of schema are just the columns to be displayed in GUI.
   * The actual data rows may contain more columns; which the tabular ignores
   * but keep in values.
   *
   * @return array
   */
  static function schema() {
    static $schema;
    if (!$schema) {
      $schema = array(
        // Cannot be composite and must be string.
        // The primary key (data) column needn't be in view_columns.
        'primary_key' => 'example_id',
        // The column used as title of a row.
        'title_column' => 'name',
        // View columns are a subset of the data columns
        // - all columns listed here must exist in all data sets that any
        // provider of the entity type delivers.
        'view_columns' => array(
          'name' => array(
            // (optional) Weight of column in row. Default zero; hidden.
            'weight' => -1,
            // Required if weight isn't zero (or undefined).
            'label' => 'Example name',
            // (optional) HTML title attribute. Column ID replacers supported.
            'hover_title' => '_VALUE_',
            // (optional) Sort view by column; asc|desc.
            'sort' => 'asc',
          ),
          'occupation' => array(
            'weight' => 1,
            'label' => 'Example occupation',
            'hover_title' => '!example_id: !name, !occupation (!updated)'
          ),
          'some_data_only' => array(
            // A column's properties list is allowed to be empty.
            // It won't be displayed by default, because it has no weight.
          ),
          'updated' => array(
            'weight' => 2,
            'label' => 'Example updated',
            'hover_title' => '_RAW_',
            'sort' => 'desc',
            // (optional) Convert instance data value to display.
            // Name of a globally accessible JS function.
            'convert' => 'Drupal.Tabular.convert.toLocaleDatetime',
          ),
        ),
        // List of button columns. Allowed to be empty.
        'buttons' => array(
          'display' => array(
            // (optional) Weight of column in row. Default zero; hidden.
            // Negative value means prepend to view columns. Position = append.
            // There can maximally be prepended 3 buttons.
            'weight' => 1,
            'label' => 'Show',
            // (optional) HTML title attribute. Column ID replacers supported.
            'hover_title' => 'Show !name',
            // (required) Onclick event handler.
            // Tokens supported: _GROUP_ID_, _WIDGET_NAME_, _PRIMARY_KEY_.
            // Get wrapped in try-catch unless already wrapped so.
            'click' => "alert(inspect.get(_GET_));",
          ),
        ),
      );
    }
    return $schema;
  }

  /**
   * Get all translated messages for this entity kind.
   *
   * @return array
   */
  static function translation() {
    static $translation;
    if (!$translation) {
      $translation = array(
        // Value widgets.
        'max_rows' => t('You cannot select more than !max examples.',
          array(), array('context' => 'tabular:example:message:max_rows')),
        'max_rows_removable' => t('You cannot select more than !max examples.<br>Please remove an example before adding a new.',
          array(), array('context' => 'tabular:example:message:max_rows_removable')),
        'value_set' => t('Selected !name.',
          array(), array('context' => 'tabular:example:message:value_set')),
        'value_removed' => t('Removed !name.',
          array(), array('context' => 'tabular:example:message:value_removed')),
        'value_truncated' => t('Removed all examples.',
          array(), array('context' => 'tabular:example:message:value_truncated')),
        // Source widgets.
        'option_none' => t('None',
          array(), array('context' => 'tabular:example:message:option_none')),
        'provider_no_results' => t('None found.',
          array(), array('context' => 'tabular:example:message:provider_no_results')),
        // Target widgets.
        'none_selected' => t('None selected.',
          array(), array('context' => 'tabular:example:message:none_selected')),
        // View type table: dynamic_table.
        'sort_by_column' => t('Sort by !label',
          array(), array('context' => 'tabular:example:message:sort_by_column')),
        'displayed_range' => t('Showing _START_ to _END_ of _MAX_ examples.',
          array(), array('context' => 'tabular:example:message:displayed_range')),
        'range_no_match' => t('No match',
          array(), array('context' => 'tabular:example:message:range_no_match')),
        'result_search_no_match' => t('No match, out of _MAX_ examples.',
          array(), array('context' => 'tabular:example:message:result_search_no_match')),
        'paginate_previous' => t('Previous',
          array(), array('context' => 'tabular:example:message:paginate_previous')),
        'paginate_next' => t('Next',
          array(), array('context' => 'tabular:example:message:paginate_next')),
      );
    }
    return $translation;
  }


  // Non-inheritance attributes and methods.------------------------------------

  /**
   * Static.
   *
   * @see tabular_example_tabular_entity_kind_provider_info()
   *
   * @param string $formBuildId
   */
  public static function ajaxPopulateByStaticProvider($formBuildId) {
    sleep(1);
    $data = self::dataExample(31, 'array');
    static::sendCacheHeaders();
    header('Content-Type: application/json; charset=utf-8');
    // Not success_data_message_response.
    echo drupal_json_encode($data);
    flush();
    exit;
  }

  /**
   * Searchable.
   *
   * @see tabular_example_tabular_entity_kind_provider_info()
   *
   * @param string $formBuildId
   * @param string string $search
   */
  public static function ajaxPopulateBySearchableProvider($formBuildId, $search = '') {
    sleep(2);
    $response = array(
      'success' => TRUE,
      'data' => array(),
    );
    if ($search !== '' && drupal_strlen($search) >= 2) {
      $search = drupal_strtolower($search);
      $data = self::dataExample(31, 'array');
      $matches = array();
      foreach ($data as $id => $list) {
        if (strpos(drupal_strtolower($list['name']), $search) !== FALSE
          || strpos(drupal_strtolower($list['occupation']), $search) !== FALSE
        ) {
          $matches[$id] = $list;
        }
      }
      $response['data'] =& $matches;
      //$response = array(
      //  'success' => FALSE,
      //  'message' => 'Please try again later.',
      //  'error_code' => self::$errorCodes['provider_gateway_timeout'],
      //);
    }
    static::sendCacheHeaders();
    header('Content-Type: application/json; charset=utf-8');
    // success_data_message_response.
    echo drupal_json_encode($response);
    flush();
    exit;
  }

  /**
   * Make some random data that fits the schema columns.
   *
   * @param integer $nRows
   * @param string $listAs
   *   Values: array|object
   *   Default: random decision.
   * @return array
   */
  public static function dataExample($nRows = 31, $listAs = '') {
    $schema = self::schema();
    $primary_key = $schema['primary_key'];
    $columns =& $schema['view_columns'];

    foreach ($columns as $name => &$props) {
      if (!isset($column['weight'])) {
        $column['weight'] = 0;
      }
    }
    uasort($columns, 'drupal_sort_weight');

    $data = array();
    // If IDs are numeric, JS will get the list as array.
    // Otherwise JS will get an object list.
    // Both types must be supported.
    if (!$listAs) {
      $frontend_receives_object_list = mt_rand(0, 1);
    }
    else {
      $frontend_receives_object_list = $listAs != 'array';
    }

    $day_ordinal = array(
      '',
      'first',
      'second',
      'third',
      'fourth',
      'fifth',
      'sixth',
      'seventh',
      'eighth',
      'ninth',
      'tenth',
      'eleventh',
      'twelth',
      'thirteenth',
      'fourteenth',
      'fifteenth',
      'sixteenth',
      'seventeenth',
      'eighteenth',
      'nineteenth',
      20 => 'twentieth',
      30 => 'thirtieth',
    );

    $previous = array();
    for ($i = 1; $i < $nRows + 1; ++$i) {
      $id_value = !$frontend_receives_object_list ? $i : Inspect::baseConvert(($i * $i) + 1000000000);
      $data[$id_value] = array($primary_key => $id_value);
      foreach ($columns as $name => $props) {
        switch ($name) {
          case 'name':
            if ($i && !($i % 5)) {
              $value = $previous['name'];
            }
            else {
              if ($i < 21 || $i == 30) {
                $value = $day_ordinal[$i];
              }
              elseif ($i < 30) {
                $value = 'twenty' . $day_ordinal[$i - 20];
              }
              else {
                $value = 'thirty' . $day_ordinal[1];
              }
              $family = $day_ordinal[$i % 10];
              $middlename = $surname = '';
              if ($i > 9) {
                for ($j = 0; $j < strlen($value); ++$j) {
                  $middlename .= chr(ord($value{$j}) + 1);
                }
              }
              for ($j = 0; $j < strlen($family); ++$j) {
                $surname .= chr(ord($family{$j}) + 1);
              }
              $previous['name'] = $value = ucfirst($value)
                . (!$middlename ? '' : (' ' . ucfirst($middlename)))
                . ' ' . ucfirst($surname);
            }
            break;
          case 'occupation':
            if ($i && !($i % 3)) {
              $value = $previous['occupation'];
            }
            else {
              switch ($i % 10) {
                case 0:
                  $value = 'Fisherman';
                  break;
                case 1:
                  $value = 'Farmer';
                  break;
                case 2:
                  $value = 'Baker';
                  break;
                case 3:
                  $value = 'Butcher';
                  break;
                case 4:
                  $value = 'Clergyman';
                  break;
                case 5:
                  $value = 'Tinkler';
                  break;
                case 6:
                  $value = 'Tailor';
                  break;
                case 7:
                  $value = 'Soldier';
                  break;
                case 8:
                  $value = 'Sailor';
                  break;
                case 9:
                  $value = 'Thief';
                  break;
                default:
                  $value = 'Spy';
              }
              $previous['occupation'] = $value;
            }
            break;
          case 'updated':
            if ($i && !($i % 6)) {
              $value = $previous['updated'];
            }
            else {
              $year = date('Y');
              $month = (int) date('m');
              if (!($i % 10)) {
                --$month;
                if (!$month) {
                  --$year;
                  $month = 12;
                }
              }
              elseif (!($i % 5)) {
                ++$month;
                if ($month == 13) {
                  ++$year;
                  $month = 1;
                }
              }
              $month = ($month > 9 ? '' : '0') . $month;
              $day = ($i > 9 ? '' : '0') . $i;
              $previous['updated'] = $value = $year . '-' . $month . '-' . $day;
            }
            break;
          default:
            $value = sha1(uniqid('', TRUE));
        }
        $data[$id_value][$name] = $value;
      }
      // Don't make more than 26 rows; otherwise object list trick will break
      // because of bad IDs (char > 122).
      if ($i == 31) {
        break;
      }
    }

    return $data;
  }

}
