/**
 * @file
 * Tabular Example (tabular) provider(s).
 */

/*jslint browser: true, continue: true, indent: 2, newcap: true, nomen: true, plusplus: true, regexp: true, white: true, ass: true*/
/*global alert: false, confirm: false, console: false*/
/*global jQuery: false, Drupal: false, Judy: false*/

(function($) {
  'use strict';

  /**
   * Namespace.
   *
   * @type object
   */
  Drupal.TabularExample = {
    example: {
      /**
       * @see Tabular.widget.source#titulate
       * @param valueRow
       * @returns {string}
       */
      titulate: function(valueRow) {
        // Beware of 'this' - the function will be executed in the context
        // of a Tabular.widget.source object.
        return valueRow.name + ' (' + valueRow.occupation + ')';
      },
      static_provider: {
        /**
         * @see Tabular.widget.source#populateAjaxPathArgs
         * @returns {Array}
         */
        populateAjaxPathArgs: function() {
          // Beware of 'this' - the function will be executed in the context
          // of a Tabular.widget.source object.
          return [];
        }
      },
      searchable_provider: {
        /**
         * Search change callback for search phrases that cannot contain space.
         * The built-in method is less crude, in that it allows spaces.
         *
         * @see Tabular.widget.source#populateSearchChange
         * @param {HTMLElement} searchElm
         *   The population_search text field input[type="text"].
         * @returns {Array}
         *   Empty if search shan't be executed (like: phrase too short).
         */
        populateSearchChange: function(searchElm) {
          // Beware of 'this' - the function will be executed in the context
          // of a Tabular.widget.source object.
          var minLength = this.populateSearchMin, v = searchElm.value;
          if (v.length >= minLength) {
            // The trim here will in effect make it impossible for user to use
            // space (because trailing space will be removed ;-).
            v = $.trim(v).
              // Any populateSearchChange function _must_ sanitize.
              replace(this.populateSearchIllegals, '');
            if (v.length >= minLength) {
              // Pass back trimmed value.
              searchElm.value = v;
              return [v];
            }
            // Pass back trimmed value.
            searchElm.value = v;
          }
          return [];
        },
        /**
         * @see Tabular.widget.source#populateAjaxPathArgs
         * @param {Array} [pathArgs]
         *   Strings.
         * @returns {Array}
         */
        populateAjaxPathArgs: function(pathArgs) {
          // Beware of 'this' - the function will be executed in the context
          // of a Tabular.widget.source object.
          return pathArgs || [];
        }
      }
    }
  };

}(jQuery));
